"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Migration = void 0;
const fs_extra_1 = __importDefault(require("fs-extra"));
const child_process_1 = __importDefault(require("child_process"));
const path_1 = __importStar(require("path"));
const bottom_line_utils_1 = require("bottom-line-utils");
const dirname_js_1 = require("./dirname/dirname.js");
const thisDir = path_1.default.dirname(path_1.default.dirname(dirname_js_1.dirname));
const run = async (command, args, options) => {
    return new Promise((resolve, reject) => {
        const proc = child_process_1.default.spawn(command, args, options);
        proc.stdout.on("data", (data) => {
            console.info(String(data).trim());
        });
        proc.stderr.on("data", (data) => {
            console.info(String(data).trim());
        });
        proc.on("error", (error) => {
            reject(error);
        });
        proc.on("close", (code) => {
            if (!code) {
                resolve();
                return;
            }
            reject(new Error(`${command} ${args.join(" ")} exited with code ${code}`));
        });
    });
};
const PKG_JSON_INDENT = 2;
const JSON_INDENT = 4;
class Migration {
    constructor(opts) {
        this._targetDir = opts.targetDir;
        this._pkg = opts.pkg;
    }
    _savePkg() {
        return fs_extra_1.default.writeFile(path_1.join(this._targetDir, "package.json"), JSON.stringify(this._pkg, null, PKG_JSON_INDENT));
    }
    get jsx() {
        return Boolean(this._pkg.libraryTemplate.jsx);
    }
    get targetDir() {
        return this._targetDir;
    }
    get pkg() {
        return this._pkg;
    }
    async setPath(objPath, value) {
        bottom_line_utils_1.set(this._pkg, objPath, value);
        await this._savePkg();
    }
    async setScript(scriptName, value) {
        await this.setPath(["scripts", scriptName], value);
    }
    async deleteScript(scriptName) {
        delete this._pkg.scripts[scriptName];
        await this._savePkg();
    }
    async upgradeScript(scriptName, oldValue, newValue) {
        this.assertScript(scriptName, oldValue, new Error("cannot update " + scriptName + " script as it was modified\n  "
            + "wanted new value:\n  `" + newValue + "`"));
        await this.setScript(scriptName, newValue);
    }
    async addDependency(name, version) {
        await this.setPath(["dependencies", name], version);
    }
    async addDevDependency(name, version) {
        await this.setPath(["devDependencies", name], version);
    }
    async upgradeDependency(name, version, defaultType = "dependencies") {
        var _a;
        const type = (_a = this.findDependency(name)) !== null && _a !== void 0 ? _a : defaultType;
        await this.setPath([type, name], version);
    }
    findDependency(name) {
        var _a, _b;
        const deps = (_a = this._pkg.dependencies) !== null && _a !== void 0 ? _a : {};
        const dev = (_b = this._pkg.devDependencies) !== null && _b !== void 0 ? _b : {};
        if (deps[name]) {
            return "dependencies";
        }
        if (dev[name]) {
            return "devDependencies";
        }
    }
    async remove(dirName) {
        await fs_extra_1.default.remove(path_1.join(this._targetDir, dirName));
    }
    async copy(sourceName, targetName = null, overwrite = true) {
        await fs_extra_1.default.copy(path_1.join(thisDir, sourceName), path_1.join(this._targetDir, targetName !== null && targetName !== void 0 ? targetName : sourceName), { overwrite });
    }
    async rename(source, target) {
        await fs_extra_1.default.rename(path_1.join(this._targetDir, source), path_1.join(this._targetDir, target));
    }
    async updateContents(file, updater) {
        const target = path_1.join(this._targetDir, file);
        const data = String(await fs_extra_1.default.readFile(target));
        const contents = await updater(data);
        await fs_extra_1.default.writeFile(target, contents);
    }
    async updateContentsJSON(file, updater) {
        const target = path_1.join(this._targetDir, file);
        const data = JSON.parse(String(await fs_extra_1.default.readFile(target)));
        const newData = await updater(data, bottom_line_utils_1.set.bind(null, data));
        await fs_extra_1.default.writeFile(target, JSON.stringify(newData !== null && newData !== void 0 ? newData : data, null, JSON_INDENT));
    }
    async updatePath(objPath, updater) {
        const current = bottom_line_utils_1.get(this._pkg, objPath);
        const newValue = updater(String(current));
        await this.setPath(objPath, newValue);
    }
    assertPath(objPath, value, error) {
        if (bottom_line_utils_1.get(this._pkg, objPath) !== value) {
            throw error;
        }
    }
    assertNoPath(objPath, error) {
        if (bottom_line_utils_1.get(this._pkg, objPath) !== undefined) {
            throw error;
        }
    }
    assertScript(scriptName, value, error) {
        this.assertPath(["scripts", scriptName], value, error);
    }
    assertNoScript(scriptName, error) {
        this.assertNoPath(["scripts", scriptName], error);
    }
    assertNoDependency(name, error) {
        if (this._pkg.dependencies && name in this._pkg.dependencies) {
            throw error;
        }
    }
    assertNoDevDependency(name, error) {
        if (this._pkg.devDependencies && name in this._pkg.devDependencies) {
            throw error;
        }
    }
    assertNoAnyDependency(name, error) {
        this.assertNoDependency(name, error);
        this.assertNoDevDependency(name, error);
    }
    assertDevDependency(name, version, error) {
        if (!this._pkg.devDependencies
            || ((version && this._pkg.devDependencies[name] !== version)
                || (!version && !this._pkg.devDependencies[name]))) {
            throw error;
        }
    }
    yarn() {
        return run("yarn", [], {
            cwd: this._targetDir,
            shell: true,
        });
    }
}
exports.Migration = Migration;
//# sourceMappingURL=Migration.js.map