#!/usr/bin/env node
"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_extra_1 = __importDefault(require("fs-extra"));
const path_1 = __importDefault(require("path"));
const empty_dir_1 = __importDefault(require("empty-dir"));
const dirname_js_1 = require("./dirname/dirname.js");
const Question_js_1 = __importDefault(require("./Question.js"));
const migrate_js_1 = require("./migrate.js");
const extractProjectName = (givenPath) => {
    if (givenPath === ".") {
        return path_1.default.basename(process.cwd());
    }
    if (path_1.default.isAbsolute(givenPath)) {
        return path_1.default.basename(givenPath);
    }
    return path_1.default.normalize(givenPath).replace(/\\/g, "/").split("/").filter(v => v !== "..").join("/");
};
const INDENT = 2;
const copyList = {
    "package.json": "template/package.json",
    "LICENSE": "template/LICENSE",
    "README.md": "template/README.md",
    "CHANGELOG.md": "template/CHANGELOG.md",
    ".npmignore": "template/.REMOVE_THIS_npmignore",
    ".gitignore": "template/.REMOVE_THIS_gitignore",
    "build-scripts": "template/build-scripts",
    "test": "test",
    ".editorconfig": ".editorconfig",
    ".eslintrc.json": ".eslintrc.json",
    "babel.config.cjs": "babel.config.cjs",
    "jest.config.cjs": "jest.config.cjs",
    "nodemon.json": "nodemon.json",
    "tsconfig.cjs.json": "tsconfig.cjs.json",
    "tsconfig.lint.json": "tsconfig.lint.json",
    "tsconfig.json": "tsconfig.json",
    "pagesconfig.json": "pagesconfig.json",
    "tutorials/Test.md": "tutorials/Test.md",
};
(async () => {
    const argsDir = process.argv[2] || ".";
    const argsJsx = (process.argv[3] === "--jsx") || false;
    const targetDir = path_1.default.resolve(argsDir);
    const thisDir = path_1.default.dirname(path_1.default.dirname(dirname_js_1.dirname));
    const thisPkg = JSON.parse(String(await fs_extra_1.default.readFile(path_1.default.join(thisDir, "package.json"))));
    console.info("Target dir:", targetDir);
    await fs_extra_1.default.ensureDir(targetDir);
    const projectNameFromPath = extractProjectName(argsDir);
    const pkgPath = path_1.default.join(targetDir, "package.json");
    const isEmpty = await empty_dir_1.default(targetDir);
    if (!isEmpty) {
        await migrate_js_1.migrate(targetDir);
        if (argsJsx) {
            await migrate_js_1.migrateJsx(targetDir);
        }
        return;
    }
    console.info("Creating new library");
    const copyProcess = Promise.all([Object.entries(copyList).map(async ([targetName, sourceName]) => {
            const target = path_1.default.join(targetDir, targetName);
            const source = path_1.default.join(thisDir, sourceName);
            await fs_extra_1.default.copy(source, target);
        })]);
    await copyProcess;
    const q = new Question_js_1.default();
    const project = await q.ask("Project name? [" + projectNameFromPath + "]");
    const version = await q.ask("Version? [0.0.1]");
    const repo = await q.ask("Repository URL? [NOT SET]");
    const jsxAnswer = argsJsx ? "y" : await q.ask("React library? [no]");
    const author = await q.ask("Author [NOT SET]");
    const defaultLicense = author || "NOT SET";
    const copy = await q.ask("Copyright (LICENSE), ie: My Name [" + defaultLicense + "]");
    q.close();
    const useJsx = jsxAnswer === "y" || jsxAnswer === "yes";
    const useCopy = copy || author;
    const useProjectName = project || projectNameFromPath;
    const targetPkg = JSON.parse(String(await fs_extra_1.default.readFile(pkgPath)));
    targetPkg.name = useProjectName;
    targetPkg.version = version || "0.0.1";
    if (repo) {
        targetPkg.repository = repo;
    }
    else {
        delete targetPkg.repository;
    }
    if (author) {
        targetPkg.author = author;
    }
    else {
        delete targetPkg.author;
    }
    targetPkg.libraryTemplate = {
        version: thisPkg.version,
        language: "typescript",
        fixDefaultForCommonJS: true,
        jsx: false,
    };
    await fs_extra_1.default.writeFile(pkgPath, JSON.stringify(targetPkg, null, INDENT));
    let lic;
    const licPath = path_1.default.join(targetDir, "LICENSE");
    const yr = new Date().getFullYear();
    lic = String(await fs_extra_1.default.readFile(licPath));
    lic = lic.replace("(c)", `(c) ${yr} ${useCopy}`);
    await fs_extra_1.default.writeFile(licPath, lic);
    await fs_extra_1.default.ensureDir(path_1.default.join(targetDir, "src"));
    if (useJsx) {
        await migrate_js_1.migrateJsx(targetDir);
    }
    console.info("");
    console.info("Done");
})().catch((e) => {
    console.error("Error happened!");
    if (e instanceof Error) {
        console.error(e);
    }
    process.exit(1);
});
//# sourceMappingURL=index.js.map