"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.migrateJsx = exports.migrate = void 0;
const path_1 = __importDefault(require("path"));
const brokenSemver = __importStar(require("semver"));
const bottom_line_utils_1 = require("bottom-line-utils");
const fs_extra_1 = __importDefault(require("fs-extra"));
const migrations_config_js_1 = require("./migrations.config.js");
const Migration_js_1 = require("./Migration.js");
const Question_js_1 = __importDefault(require("./Question.js"));
const semver = {
    gte: (brokenSemver.gte || brokenSemver.default.gte),
};
const applyMigrations = async ({ migration, migrations, updateVersion }) => {
    let skipped = 0;
    const err = (e) => {
        skipped++;
        console.warn("✖", e.message);
    };
    for (let i = 0; i < migrations.length; i++) {
        const migrationConfig = migrations[i];
        console.info(`Upgrading [${migrationConfig.version} -> ${migrationConfig.nextVersion}]`);
        for (let j = 0; j < migrationConfig.steps.length; j++) {
            const step = migrationConfig.steps[j];
            if ("jsx" in step && step.jsx !== migration.jsx) {
                continue;
            }
            console.info();
            console.info(`- ${step.name}`);
            await step.fn(migration).then(() => { console.warn("✔ ok"); }, err);
        }
    }
    const migCfg = bottom_line_utils_1.last(migrations);
    if (migCfg && updateVersion) {
        await migration.setPath("libraryTemplate.version", migCfg.nextVersion);
    }
    return skipped;
};
const aggressiveMessage = async (migrations) => {
    const aggressive = migrations.map(m => m.aggresive).filter(Boolean);
    if (!aggressive.length) {
        return true;
    }
    console.info("");
    console.info("WARNING: Aggressive upgrade detected!");
    console.info("Aggressive upgrade means that customizations applied over previously-generated library may be lost "
        + "or that library won't work after upgrading without further changes to the code. Please verify this list "
        + "of POTENTIAL POSSIBLE issues and accept the upgrade progress:");
    console.info(aggressive.map(a => "- " + a.trim()).join("\n"));
    console.info("");
    const q = new Question_js_1.default();
    const accept = await q.ask("Do you want to continue? (y/n) [n]");
    q.close();
    return accept.startsWith("y");
};
const prepareMigrate = async (targetDir) => {
    let pkg, lang, ver, rethrow = false;
    const pkgPath = path_1.default.join(targetDir, "package.json");
    try {
        pkg = JSON.parse(String(await fs_extra_1.default.readFile(pkgPath)));
        ver = bottom_line_utils_1.get(pkg, ["libraryTemplate", "version"]);
        if (!ver) {
            throw new Error("No version");
        }
        lang = bottom_line_utils_1.get(pkg, ["libraryTemplate", "language"]);
        if (lang !== "typescript") {
            rethrow = true;
            throw new Error("Migration from js-library-template is currently not supported");
        }
    }
    catch (e) {
        if (rethrow) {
            throw e;
        }
        throw new Error("Target directory is not empty, no supported library found to upgrade.");
    }
    return {
        pkg, ver,
    };
};
const migrate = async (targetDir) => {
    const { pkg, ver } = await prepareMigrate(targetDir);
    const migration = new Migration_js_1.Migration({ targetDir, pkg });
    const migrations = migrations_config_js_1.migrationsConfig.filter(migrationConfig => {
        return semver.gte(migrationConfig.version, ver);
    });
    const versions = migrations.map(m => m.nextVersion);
    const cont = await aggressiveMessage(migrations);
    if (!cont) {
        console.info("Upgrade stopped.");
        return;
    }
    if (!migrations.length) {
        console.info("The project is up to date with current ts-library-template.");
        return;
    }
    console.info(`${migrations.length} updates to apply, versions: ${versions.join(", ")}`);
    const updateVersion = true;
    const skipped = await applyMigrations({ migration, migrations, updateVersion });
    console.info();
    if (skipped) {
        console.info(`Skipped ${skipped} upgrades. Please perform them manually if needed.`);
    }
    console.info("Upgrading finished.");
};
exports.migrate = migrate;
const migrateJsx = async (targetDir) => {
    const { pkg } = await prepareMigrate(targetDir);
    if (pkg.libraryTemplate.jsx) {
        return;
    }
    const migration = new Migration_js_1.Migration({ targetDir, pkg });
    const migrations = [migrations_config_js_1.jsxMigration];
    const updateVersion = false;
    const skipped = await applyMigrations({ migration, migrations, updateVersion });
    console.info();
    if (skipped) {
        console.info(`Skipped ${skipped} upgrades. Please perform them manually if needed.`);
    }
    console.info("Upgrading finished.");
};
exports.migrateJsx = migrateJsx;
//# sourceMappingURL=migrate.js.map